/* set_filt.c */
/* Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008 Joseph Rosevear */

/*
This file is part of The SAM Kernel.

The SAM Kernel is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or (at
your option) any later version.

The SAM Kernel is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/



/*
050309 the "=()" that set produced in Slack has changed to " ()".  This
affects both set_filt.c and uns_filt.c.  So I'm going to combine the
two into one program and give an option for version.

I'll use $1 for version (1 for "=()", and 2 for " ()").

$2 will be "set" or "unset", although only the first character will be
read.

Some explanation about bef_equal.  It was fst_equal.  ("before equal" and
"first equal".)  I made this change, because it helps to clarify what this
variable does.

About version 1 and version 2.  This refers to two ways in which this code
can function.  Version 1 is for a set command that gives "=()" and version
2 is for a set command that gives " ()".  I found that Slack 7.1 is version
1 and Slack 10.0 is version 2.

The code in this that deals with version 2 does not check bef_equal.  I
don't think it needs to, because it was there for cases like this

   fred=far=()

This is an issue for version 1.  But for version 2 the corresponding
situation is

   fred=far ()
   
which shouldn't happen except inside a string (delimited by ' marks). 
In a string the problem is covered by the use of the "instring" check.
So, that's why I left out the bef_equal check for version 2.  If I find
I need it, then I'll put it in.

Hmmm.  I wonder if I need the checks for "lower".  I can't remember why
they are there.  This gives wrong SAM behavior when functions have been
made that start with a capital letter. (I tried.)  So I'll take this out.

was

   if ((bef_equal && !instring && lower) && (argv[1][0] == '1'))

now

   if ((bef_equal && !instring) && (argv[1][0] == '1'))
   
for '='.  I'll make corresponding chang for ' '.

050318 I made some more changes.  It turns out that there was a reason
for the use of lower questioned above.  The code was faulty.  I was
using case to distinguish between A=() (or "a=()") the, variable
definition, and A=() (or "a=()"), the beginning of a function.  (This
discrimination was important, because the variable dump was set to 1 on
the first occurence of a function.) This worked pretty well, because
the O.S. used A=() for variable definions (upper case).  There were a
few upper case variables set to ().  So all I had to do was to (1)
avoid use of functions that started with a capital, and (2) avoid
setting a lower case variable to ().  (Violating (1) for the first
encountered function would cause the triggering of dump to be delayed
(maybe serious).  Violating (2) would cause dump to be triggered early
(serious).) This done, the distinction was effective.

However, I couldn't live with the limitations or the risks.  It was bad
code, anyway.  So, I changed the code to something not much better.  I
no longer use case to discriminate as I did above.  dump is still set
to 1 on the first occurence of a function, but functions are recognized
(for sam_set_version = 1) by c,d,e,h equal to =,(,),{.  Note f and g
are space and newline.  For sam_set_version = 2, I can still use just
c,d,e as before, so I do.

So the use of lower to aid in setting dump is gone.  lower is still
used (along with okp) as before to decide whether to print buffer. 
This brings up the subject of okp.

I made some changes to how okp is set.  Before the test was too easy. 
I changed the test so that only "PATH=" or "PS1=" (both must start in
column 1) can cause okp to be set to 1.

Some information: okp means "OK P" which refers to PATH and PS1, the
only two variables that start with upper case that SAM sets.  SAM does
not unset any such variables.  Here, using case works.  The rule is
"(1) Variables that start with upper case work fine in SAM, but they
are neither set nor unset by the synthetic shell process; (2) An
exception to the previous rule is that the variables PATH and PS1 are
set by the synthetic shell, but not unset, and therefore behave
something like lower case variables, but not quite; The difference is
that these variables experience reverse inheritance when going back to
a previous synthetic shell when they were not defined in that previous
shell.  However you're not likely to see the reverse inheritance,
because PATH and PS1 are always set."

070506

Having trouble.  Am getting "Error. Buffer size exceeded." message.  So
I increased BUFFSIZE below from 1000 to 2000.  Still getting message. 
Setting to 4000.  Setting to 10000.  I found the problem, sort of.  I
had used a "'" character as an apostrophe inside a web page title in my
colpg.sam file.  I don't know how that translated into a problem here,
but removing the apostrophe allowed me to go back to 1000 for BUFFIZE. 
The use of 10000 was causing an "Unmatched '" error message.

080110

Having trouble with "Buffer size exceeded" message again.  I tried a
BUFFSIZE of 10000 and that fixed it, but that is too big all at once. 
So I tried 2000 (was 1000), and that fixed it.  I also changed two
error messages to reveal what program they are coming from.

*/

#include <stdio.h>

#define BUFFSIZE 2000

static int d, e, f, g, h;

int getahead() {

   int c;
   extern int d, e, f, g, h;

   static int first = 1;

   if (first) {
      c = getchar();
      d = getchar();
      e = getchar();
      f = getchar();
      g = getchar();
      h = getchar();
      first = 0;
   } else {
      c = d;
      d = e;
      e = f;
      f = g;
      g = h;
      h = getchar();
   }
   return(c);
}

int main(int argc, char* argv[]) {

   int c, dump, bef_equal, index, instring, lower, okp;
   extern int d, e, f, g, h;
   
   char buffer[BUFFSIZE];
   
   if (argc != 3) {
      fprintf(stderr, "Usage: set_filt.exe <version> <\"set\" or \"unset\">\n");
      return(1);
   }

   okp=0;
   lower=0;
   instring=0;
   index=0;
   dump=0;
   bef_equal=1;
   while ((c = getahead()) != EOF ) {
      
      if (!dump) buffer[index++] = c;
      
      if (index >= BUFFSIZE-1) {
         fprintf(stderr, "set_filt.exe:  Error.  Buffer size exceeded.\n");
         return(1);
      }
      
      /* this allows only lower case variables */
      if (c > 96 && index==1) lower=1;

      switch(argv[2][0]) {
      case 's': 
         break;
      case 'u':
         break;
      default:
         fprintf(stderr, "Usage: set_filt.exe <version> <\"set\" or \"unset\">\n");
         return(1);
      }
       
      switch(c) {
      case '\n':
         if (!instring) {
            buffer[index++] = '\0';
            
            if (!dump) {
               if (lower || okp) printf("%s", buffer);
               instring = 0;
               index = 0;
               bef_equal=1;
               lower=0;
               okp=0;
            }
         }
         break;
      case '=':
         if (d == '(' && e == ')' && h == '{') {
            if ((bef_equal && !instring) && (argv[1][0] == '1'))
               dump=1;
         }
         bef_equal=0;
         break;
      case ' ':
         if (d == '(' && e == ')') {
            if ((!instring) && (argv[1][0] == '2'))
               dump=1;
         }
         break;
      case '\'':
         if (!instring)
            instring=1;
         else
            instring=0;
         break;
      case 'P':
         if (d == 'A' && e == 'T' && f == 'H' && g == '=') {
            if ((index == 1) && (argv[2][0] == 's'))
               okp=1;
         } else if (d == 'S' && e == '1' && f == '=') {
            if ((index == 1) && (argv[2][0] == 's'))
               okp=1;
         }
         break;
      } 
   }
   
   /* Should be at end of file, and should not be instring */
   if (instring) {
      fprintf(stderr, "set_filt.exe:  Error.  Unmatched \'");
      return(1);
   }
   
   return(0);
}
